/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 * @defgroup pin_change_int_example_main main.c
 * @{
 * @ingroup pin_change_int_example
 * @brief Pin Change Interrupt Example Application main file.
 *
 * This file contains the source code for a sample application using interrupts triggered by GPIO pins.
 *
 */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "app_error.h"
#include "app_uart.h"
#include "boards.h"
#include "nrf.h"
#include "nrf_drv_gpiote.h"
#include "nrf_log.h"
#include "nrf_log_default_backends.h"
#include "nrf_log_ctrl.h"
#include "nrf_delay.h"
#include "nrf_uart.h"
#include "nrf_drv_clock.h"
#include "app_timer.h"
#include "drv_rtc.h"
#include "nrfx_uart.h"

#define RXD_GPIOTE_PIN 26
#define TXD_GPIOTE_PIN 27

volatile bool myRX_DONE_FLAG=false;
volatile bool myGPIOTE_EVENT_FLAG=false;
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 256                         /**< UART RX buffer size. */
#define UART_HWFC APP_UART_FLOW_CONTROL_DISABLED
static uint8_t rxdata_array[10]={0};
volatile uint8_t myindex = 0;
APP_TIMER_DEF(m_ctrl_uart);

app_uart_comm_params_t const comm_params =
{
    .rx_pin_no    = RXD_GPIOTE_PIN,
    .tx_pin_no    = TXD_GPIOTE_PIN,
    .rts_pin_no   = RTS_PIN_NUMBER,
    .cts_pin_no   = CTS_PIN_NUMBER,
    .flow_control = APP_UART_FLOW_CONTROL_DISABLED,
    .use_parity   = false,
    .baud_rate    = NRF_UART_BAUDRATE_115200
};

static void in_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    myGPIOTE_EVENT_FLAG=true;
    for(int i=0;i<50;i++);
}
static void lfclk_config(void)
{
    ret_code_t err_code;
    err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);
    nrf_drv_clock_lfclk_request(NULL);
}


static void gpio_init(void)
{
    ret_code_t err_code;
    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(false);
    in_config.pull = NRF_GPIO_PIN_PULLUP; //Pin pull-up resistor enabled.
    err_code = nrf_drv_gpiote_in_init(RXD_GPIOTE_PIN, &in_config, in_pin_handler);
    APP_ERROR_CHECK(err_code);
    nrf_drv_gpiote_in_event_enable(RXD_GPIOTE_PIN, true);
	  myGPIOTE_EVENT_FLAG=false;
}

void uart_event_handle(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        case APP_UART_DATA_READY://Indicating that UART data(1Byte)received. The data is available in the FIFO and can be fetched using @ref app_uart_get.
            UNUSED_VARIABLE(app_uart_get(&rxdata_array[myindex]));
            myindex++;
            //APP_ERROR_CHECK(app_timer_start(m_ctrl_uart,APP_TIMER_TICKS(800),NULL));
            break;
        case APP_UART_COMMUNICATION_ERROR:
            //NRF_LOG_INFO("COMMUNICATION_ERROR:%x",p_event->data.error_communication);
            break;
        case APP_UART_FIFO_ERROR:
            //NRF_LOG_INFO("FIFO_ERROR:%x",p_event->data.error_code);
            break;
        default:
            break;
    }
}

static void gpio_uninit(void)
{
    nrf_drv_gpiote_in_event_disable(RXD_GPIOTE_PIN);
    nrf_drv_gpiote_in_uninit(RXD_GPIOTE_PIN);
    nrf_drv_gpiote_uninit();
}

static void uart_init(void)
{
    uint32_t    err_code;
    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_event_handle,
                       APP_IRQ_PRIORITY_LOWEST,
                       err_code);
    APP_ERROR_CHECK(err_code);
}

static void uart_timer_handler(void * p_context)
{
    myRX_DONE_FLAG=true;
    NRF_LOG_INFO("uart receive timeout");
}

static void timers_init(void)
{
    ret_code_t err_code = app_timer_init();
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_create(&m_ctrl_uart, APP_TIMER_MODE_SINGLE_SHOT, uart_timer_handler);
    APP_ERROR_CHECK(err_code);
}


/**
 * @brief Function for application main entry.
 */
int main(void)
{
    //ret_code_t err_code;
    bsp_board_init(BSP_INIT_LEDS);
    lfclk_config();
    timers_init();
    APP_ERROR_CHECK(NRF_LOG_INIT(NULL));
    NRF_LOG_DEFAULT_BACKENDS_INIT();
    gpio_init();
    NRF_LOG_INFO("Main");

    while(true){
        __WFE();
			  __SEV();
        if(myGPIOTE_EVENT_FLAG){
            bsp_board_led_on(BSP_BOARD_LED_0); 
            APP_ERROR_CHECK(app_timer_start(m_ctrl_uart,APP_TIMER_TICKS(800),NULL));
            gpio_uninit();
            uart_init();
            while(!myRX_DONE_FLAG);
            myRX_DONE_FLAG=false;
            //APP_ERROR_CHECK(app_timer_stop(m_ctrl_uart));
            bsp_board_led_off(BSP_BOARD_LED_0);
            APP_ERROR_CHECK(app_uart_close());
            gpio_init();
            if(myindex<=4){
                NRF_LOG_INFO("rxdata_array[3]=%d",rxdata_array[3]);
            }else{
                NRF_LOG_INFO("Not receice right data in 0.8S, index=%d",myindex)
            }
            NRF_LOG_FLUSH();  
            myindex=0;
            memset(rxdata_array,0x0,5);
        }
        __SEV();
        __WFE();
    }
}


/** @} */
